@@ -21,7 +21,7 @@ use std::ops::Deref;
 use std::path::Path;
 use strip;
 use types::{EventType, ParseResult, Type, TypeStruct};
-use utils::{join, loop_over_files, write_comment, write_file};
+use utils::{join, loop_over_files, remove_macro_parent, write_comment, write_file};
 
 type Infos = HashMap<Option<String>, Vec<(Option<TypeStruct>, Vec<String>)>>;
 
@@ -41,13 +41,22 @@ fn gen_indent_from(from: &str) -> String {
     String::new()
 }
 
+/// Returns `true` in case a "// rustdoc-stripper-ignore-next-stop" was inserted.
 fn regenerate_comment(
     is_file_comment: bool,
     position: usize,
     indent: usize,
     comment: &str,
     original_content: &mut Vec<String>,
-) {
+    need_check_ignore_doc_comment: bool,
+) -> bool {
+    let mut need_to_add_ignore_next_comment_stop = false;
+    if need_check_ignore_doc_comment && position > 0 {
+        let prev = original_content[position - 1].trim();
+        if strip::DOC_COMMENT_ID.iter().any(|d| prev.starts_with(d)) {
+            need_to_add_ignore_next_comment_stop = true;
+        }
+    }
     let is_empty = comment.trim().is_empty();
     let read_indent = if is_file_comment {
         gen_indent(indent)
@@ -62,9 +71,16 @@ fn regenerate_comment(
             &read_indent,
             if is_file_comment { "//!" } else { "///" },
             if is_empty { "" } else { " " },
-            if is_empty { "" } else { &comment }
+            if is_empty { "" } else { comment }
         ),
     );
+    if need_to_add_ignore_next_comment_stop {
+        original_content.insert(
+            position,
+            format!("{}{}", &read_indent, strip::IGNORE_NEXT_COMMENT_STOP,),
+        );
+    }
+    need_to_add_ignore_next_comment_stop
 }
 
 #[allow(clippy::useless_let_if_seq)]
@@ -76,27 +92,38 @@ fn get_corresponding_type(
     original_content: &mut Vec<String>,
     ignore_macros: bool,
 ) -> Option<usize> {
+    if to_find.is_none() {
+        return None;
+    }
+    let to_find = to_find.as_ref().unwrap();
     let mut pos = 0;
 
     while pos < elements.len() {
-        if match (&elements[pos].0, to_find) {
-            (&Some(ref a), &Some(ref b)) => {
-                let ret = a == b;
+        if match elements[pos].0 {
+            Some(ref a) => {
+                // is true if a is a top level Type, or if is inside a macro and Type and name match
+                /* The result is that if there is a struct defined inside a macro,
+                the documentation (if it has) of that struct will be written inside the macro. */
+                let ret = a == to_find || {
+                    let mut tmp = to_find.clone();
+                    remove_macro_parent(&mut tmp);
+                    *a == tmp
+                };
 
                 // to detect variants
                 if !ret
-                    && b.ty == Type::Unknown
-                    && b.parent.is_some()
+                    && to_find.ty == Type::Unknown
+                    && to_find.parent.is_some()
                     && a.parent.is_some()
-                    && a.parent == b.parent
+                    && a.parent == to_find.parent
                 {
-                    if match b.parent {
+                    if match to_find.parent {
                         Some(ref p) => {
                             p.ty == Type::Struct || p.ty == Type::Enum || p.ty == Type::Use
                         }
                         None => false,
                     } {
-                        let mut tmp = b.clone();
+                        let mut tmp = to_find.clone();
                         tmp.ty = Type::Variant;
                         a == &tmp
                     } else {
@@ -123,23 +150,25 @@ fn get_corresponding_type(
                     line -= 1;
                 }
             }
-            for comment in (&elements[pos].1)
-                .iter()
-                .skip(if file_comment { 1 } else { 0 })
-            {
+            let mut first = true;
+            for comment in elements[pos].1.iter().skip(usize::from(file_comment)) {
                 let depth = if let Some(ref e) = elements[pos].0 {
                     e.get_depth(ignore_macros)
                 } else {
                     0
                 };
-                regenerate_comment(
+                if regenerate_comment(
                     file_comment,
                     line + *decal,
                     depth + 1,
-                    &comment,
+                    comment,
                     original_content,
-                );
+                    first,
+                ) {
+                    *decal += 1;
+                }
                 *decal += 1;
+                first = false;
             }
             return Some(pos);
         }
@@ -192,10 +221,10 @@ pub fn regenerate_comments(
 fn check_if_regen(it: usize, parse_result: &ParseResult, ignore_doc_commented: bool) -> bool {
     ignore_doc_commented
         && it > 0
-        && match parse_result.event_list[it - 1].event {
-            EventType::Comment(_) | EventType::FileComment(_) => true,
-            _ => false,
-        }
+        && matches!(
+            parse_result.event_list[it - 1].event,
+            EventType::Comment(_) | EventType::FileComment(_)
+        )
 }
 
 fn do_regenerate(
@@ -263,7 +292,7 @@ fn do_regenerate(
 
                     if !check_if_regen(it, parse_result, ignore_doc_commented) {
                         if let Some(l) = get_corresponding_type(
-                            &elements,
+                            elements,
                             &tmp,
                             parse_result.event_list[it].line,
                             &mut decal,
@@ -287,7 +316,7 @@ fn do_regenerate(
 
                         if !check_if_regen(it, parse_result, ignore_doc_commented) {
                             if let Some(l) = get_corresponding_type(
-                                &elements,
+                                elements,
                                 &cc,
                                 parse_result.event_list[it].line,
                                 &mut decal,
@@ -423,7 +452,7 @@ pub fn regenerate_doc_comments(
     loop_over_files(
         directory.as_ref(),
         &mut |w, s| regenerate_comments(w, s, &mut infos, ignore_macros, ignore_doc_commented),
-        &ignores,
+        ignores,
         verbose,
     );
     save_remainings(&infos, comment_file);
@@ -450,11 +479,7 @@ fn sub_erase_macro_path(ty: Option<Box<TypeStruct>>, is_parent: bool) -> Option<
 }
 
 fn erase_macro_path(ty: Option<TypeStruct>) -> Option<TypeStruct> {
-    if let Some(t) = ty {
-        Some(*sub_erase_macro_path(Some(Box::new(t)), false).unwrap())
-    } else {
-        None
-    }
+    ty.map(|t| *sub_erase_macro_path(Some(Box::new(t)), false).unwrap())
 }
 
 pub fn parse_cmts<S, I>(lines: I, ignore_macros: bool) -> Infos
@@ -477,12 +502,12 @@ where
     // The "*" entries are to be applied regardless of file name
     #[allow(clippy::option_option)]
     fn line_file(line: &str) -> Option<Option<String>> {
-        if line.starts_with(FILE) {
-            let name = &line[FILE.len()..].replace(END_INFO, "");
+        if let Some(after) = line.strip_prefix(FILE) {
+            let name = after.replace(END_INFO, "");
             if name == "*" {
                 Some(None)
             } else {
-                Some(Some(name.to_owned()))
+                Some(Some(name))
             }
         } else {
             None
@@ -503,7 +528,7 @@ where
                         comments: vec![],
                     }
                 } else {
-                    panic!("Unrecognized format");
+                    panic!("Unrecognized format on line: `{}`", line.deref());
                 }
             }
             State::File {
