@@ -20,23 +20,27 @@ use std::process::exit;
 use types::{EventInfo, EventType, ParseResult, Type, TypeStruct};
 use utils::{join, write_comment, write_file, write_file_comment};
 
-const STOP_CHARACTERS: &[char] = &['\t', '\n', '\r', '<', '{', ':', ';', '!', '('];
+const STOP_CHARACTERS: &[char] = &['\t', '\n', '\r', '<', '{', ':', ';', '!', '(', ','];
 const COMMENT_ID: &[&str] = &["//", "/*"];
-const DOC_COMMENT_ID: &[&str] = &["///", "/*!", "//!"];
-const IGNORE_NEXT_COMMENT: &str = "// rustdoc-stripper-ignore-next";
+pub(crate) const DOC_COMMENT_ID: &[&str] = &["///", "/*!", "//!", "/**"];
+pub(crate) const IGNORE_NEXT_COMMENT: &str = "// rustdoc-stripper-ignore-next";
+pub(crate) const IGNORE_NEXT_COMMENT_STOP: &str = "// rustdoc-stripper-ignore-next-stop";
 
 fn move_to(words: &[&str], it: &mut usize, limit: &str, line: &mut usize, start_remove: &str) {
-    if words[*it][start_remove.len()..].contains(&limit) {
+    if words[*it][start_remove.len()..].contains(limit) {
         return;
     }
     *it += 1;
-    while *it < words.len() && !words[*it].contains(limit) {
-        if words[*it] == "\n" {
+    while let Some(&word) = words.get(*it) {
+        if words[*it].contains(limit) {
+            break;
+        }
+        if word == "\n" {
             *line += 1;
         }
         *it += 1;
     }
-    if *it < words.len() && words[*it] == "\n" {
+    if let Some(&"\n") = words.get(*it) {
         *line += 1;
     }
 }
@@ -91,21 +95,14 @@ pub fn add_to_type_scope(
                 Some(tmp)
             }
         },
-        None => match *e {
-            Some(ref t) => Some(t.clone()),
-            _ => None,
-        },
+        None => e.as_ref().cloned(),
     }
 }
 
 pub fn type_out_scope(current: &Option<TypeStruct>) -> Option<TypeStruct> {
-    match *current {
-        Some(ref c) => match c.parent {
-            Some(ref p) => Some(p.deref().clone()),
-            None => None,
-        },
-        None => None,
-    }
+    current
+        .as_ref()
+        .and_then(|c| c.parent.as_ref().map(|p| p.deref().clone()))
 }
 
 fn get_mod(current: &Option<TypeStruct>) -> bool {
@@ -135,14 +132,10 @@ fn get_three_parts<'a>(
     stop: &str,
 ) -> (String, String, &'a str) {
     if let Some(pos) = after.find(stop) {
-        let extra = if stop != "\n" {
-            stop.len()
-        } else {
-            0
-        };
+        let extra = if stop != "\n" { stop.len() } else { 0 };
         (
             before.to_owned(),
-            format!("{} {}", comment_sign, &after[0..pos]),
+            format!("{} {}", comment_sign, &after[..pos]),
             &after[pos + extra..],
         )
     } else {
@@ -154,17 +147,28 @@ fn get_three_parts<'a>(
     }
 }
 
-fn check_if_should_be_ignored(text: &str, doc_comment: &str) -> bool {
-    let end = if text.len() <= doc_comment.len() {
-        text.len() - 1
-    } else {
-        text.len() - doc_comment.len()
-    };
-    if let Some(end_pos) = text[..end].rfind('\n') {
-        if let Some(start_pos) = text[..end_pos].rfind('\n') {
-            return text[start_pos..end_pos]
-                .trim_start()
-                .starts_with(IGNORE_NEXT_COMMENT);
+fn check_if_should_be_ignored(text: &str) -> bool {
+    let mut ignore_until_multi_end = false;
+    for line in text.split('\n').rev() {
+        let line = line.trim();
+        if !ignore_until_multi_end && !line.starts_with("//") {
+            if line.trim().ends_with("*/") {
+                ignore_until_multi_end = !line.starts_with("/**") && !line.starts_with("/*!");
+                continue;
+            }
+        } else if line.starts_with("/*") {
+            ignore_until_multi_end = false;
+            continue;
+        }
+        if !ignore_until_multi_end {
+            if line == IGNORE_NEXT_COMMENT_STOP {
+                return false;
+            } else if line == IGNORE_NEXT_COMMENT {
+                return true;
+            }
+            if !line.starts_with("///") && !line.starts_with("//!") {
+                break;
+            }
         }
     }
     false
@@ -173,24 +177,24 @@ fn check_if_should_be_ignored(text: &str, doc_comment: &str) -> bool {
 fn find_one_of<'a>(comments: &[&str], doc_comments: &[&str], text: &'a str) -> BlockKind<'a> {
     let mut last_pos = 0;
 
-    let tmp_text = &text[last_pos..];
-    if let Some(pos) = tmp_text.find('/') {
-        let tmp_text = &tmp_text[pos..];
+    let mut tmp_text = &text[last_pos..];
+    while let Some(pos) = tmp_text.find('/') {
+        tmp_text = &tmp_text[pos..];
         last_pos += pos;
         for com in doc_comments {
-            if tmp_text.starts_with(com) {
+            if let Some(after) = tmp_text.strip_prefix(com) {
                 if &com[1..2] == "*" {
                     return BlockKind::DocComment(get_three_parts(
-                        &text[0..last_pos],
+                        &text[..last_pos],
                         com,
-                        &text[last_pos + com.len()..],
+                        after,
                         "*/",
                     ));
                 } else {
                     return BlockKind::DocComment(get_three_parts(
-                        &text[0..last_pos],
+                        &text[..last_pos],
                         com,
-                        &text[last_pos + com.len()..],
+                        after,
                         "\n",
                     ));
                 }
@@ -202,36 +206,45 @@ fn find_one_of<'a>(comments: &[&str], doc_comments: &[&str], text: &'a str) -> B
                     return BlockKind::Comment(get_three_parts(
                         &text[0..last_pos],
                         "",
-                        &text[last_pos..],
+                        tmp_text,
                         "*/",
                     ));
                 } else {
                     return BlockKind::Comment(get_three_parts(
                         &text[0..last_pos],
                         "",
-                        &text[last_pos..],
+                        tmp_text,
                         "\n",
                     ));
                 }
             }
         }
+        if !tmp_text.is_empty() {
+            tmp_text = &tmp_text[1..];
+            last_pos += 1;
+        } else {
+            break;
+        }
     }
     BlockKind::Other(text)
 }
 
 fn transform_code(code: &str) -> String {
-    code.replace("{", " { ")
-        .replace("}", " } ")
-        .replace(":", " : ")
+    code.replace('{', " { ")
+        .replace('}', " } ")
+        .replace(':', " : ")
         .replace(" :  : ", "::")
         .replace("*/", " */")
-        .replace("\n", " \n ")
+        .replace('\n', " \n ")
         .replace("!(", " !! (")
         .replace("!  {", " !? {")
-        .replace(",", ", ")
-        .replace("(", " (")
+        .replace(',', ", ")
+        .replace('(', " (")
+        .replace('"', " \"")
 }
 
+// Replaces lines that should be removed (doc comments mostly) with empty lines to keep a working
+// line match.
 fn clean_input(s: &str) -> String {
     let mut ret = String::new();
     let mut text = s;
@@ -242,31 +255,18 @@ fn clean_input(s: &str) -> String {
                 break;
             }
             BlockKind::DocComment((before, doc_comment, after))
-                if !check_if_should_be_ignored(&s[..s.len() - after.len()], &doc_comment) =>
+                if !check_if_should_be_ignored(&s[..s.len() - after.len()]) =>
             {
                 ret.push_str(&transform_code(&before));
                 ret.push_str(&doc_comment);
                 after
             }
-            BlockKind::DocComment((before, comment, after)) => {
+            BlockKind::DocComment((before, doc_comment, after)) => {
                 ret.push_str(&transform_code(&before));
-                for _ in 0..comment.split('\n').count() - 1 {
+                for _ in 0..doc_comment.split('\n').count() - 1 {
                     ret.push_str(" \n ");
                 }
-                // If this is an inline comment, we need to discard the whole block as well.
-                if &comment[1..2] != "*" {
-                    let mut extra = 1;
-                    let doc_comment = &comment[..3];
-                    for line in after.split('\n').skip(1) {
-                        if !line.trim_start().starts_with(doc_comment) {
-                            break;
-                        }
-                        extra += line.len() + 1; // + 1 is for the backline
-                    }
-                    &after[extra..]
-                } else {
-                    after
-                }
+                after
             }
             BlockKind::Comment((before, comment, after)) => {
                 ret.push_str(&transform_code(&before));
@@ -292,10 +292,7 @@ fn clear_events(mut events: Vec<EventInfo>) -> Vec<EventInfo> {
                     waiting_type = Some(t.clone());
                     false
                 } else if let Some(ref parent) = current {
-                    match parent.ty {
-                        Type::Struct | Type::Enum => false,
-                        _ => true,
-                    }
+                    !matches!(parent.ty, Type::Struct | Type::Enum)
                 } else {
                     true
                 }
@@ -320,6 +317,16 @@ fn clear_events(mut events: Vec<EventInfo>) -> Vec<EventInfo> {
     events
 }
 
+fn remove_stop_chars(s: &str) -> String {
+    let mut s = s.to_owned();
+    for c in STOP_CHARACTERS {
+        if s.contains(*c) {
+            s = s.replace(&c.to_string(), "");
+        }
+    }
+    s
+}
+
 #[allow(clippy::useless_let_if_seq)]
 fn build_event_inner(
     it: &mut usize,
@@ -333,27 +340,22 @@ fn build_event_inner(
     let mut waiting_for_macro = false;
     while *it < words.len() {
         match words[*it] {
-            c if c.starts_with('\"') => move_to(&words, it, "\"", line, "\""),
-            c if c.starts_with("b\"") => move_to(&words, it, "\"", line, "b\""),
+            c if c.starts_with('"') => move_to(words, it, "\"", line, "\""),
+            c if c.starts_with("b\"") => move_to(words, it, "\"", line, "b\""),
             // c if c.starts_with("'") => move_to(&words, it, "'", line),
             c if c.starts_with("r#") => {
-                let end = c
-                    .split("#\"")
-                    .next()
-                    .unwrap()
-                    .replace("\"", "")
-                    .replace("r", "");
-                move_to(&words, it, &format!("\"{}", end), line, "r#");
+                let end = c.split("#\"").next().unwrap().replace(['"', 'r'], "");
+                move_to(words, it, &format!("\"{}", end), line, "r#");
             }
-            "///" => {
+            "///" | "///\n" => {
                 comment_lines.push(*line);
                 event_list.push(EventInfo::new(
                     *line,
                     EventType::Comment(b_content[*line].to_owned()),
                 ));
-                move_to(&words, it, "\n", line, "");
+                move_to(words, it, "\n", line, "");
             }
-            "//!" => {
+            "//!" | "//!\n" => {
                 comment_lines.push(*line);
                 event_list.push(EventInfo::new(
                     *line,
@@ -362,11 +364,11 @@ fn build_event_inner(
                 if *line + 1 < b_content.len() && b_content[*line + 1].is_empty() {
                     comment_lines.push(*line + 1);
                 }
-                move_to(&words, it, "\n", line, "");
+                move_to(words, it, "\n", line, "");
             }
-            "/*!" => {
+            "/*!" | "/*!\n" => {
                 let mark = *line;
-                move_until(&words, it, "*/", line);
+                move_until(words, it, "*/", line);
                 for (pos, s) in b_content.iter().enumerate().take(*line).skip(mark) {
                     comment_lines.push(pos);
                     event_list.push(EventInfo::new(*line, EventType::FileComment(s.to_owned())));
@@ -378,27 +380,48 @@ fn build_event_inner(
                     removed = true;
                 }
                 event_list.push(EventInfo::new(
-                    *line,
+                    mark,
                     EventType::FileComment("*/".to_owned()),
                 ));
                 if removed {
                     event_list.push(EventInfo::new(*line, EventType::FileComment("".to_owned())));
                 }
             }
+            "/**" | "/**\n" => {
+                let mark = *line;
+                move_until(words, it, "*/", line);
+                for (pos, s) in b_content.iter().enumerate().take(*line).skip(mark) {
+                    comment_lines.push(pos);
+                    event_list.push(EventInfo::new(*line, EventType::Comment(s.to_owned())));
+                }
+                comment_lines.push(*line);
+                let mut removed = false;
+                if *line + 1 < b_content.len() && b_content[*line + 1].is_empty() {
+                    comment_lines.push(*line + 1);
+                    removed = true;
+                }
+                event_list.push(EventInfo::new(
+                    mark,
+                    EventType::FileComment("*/".to_owned()),
+                ));
+                if removed {
+                    event_list.push(EventInfo::new(*line, EventType::Comment("".to_owned())));
+                }
+            }
             "use" | "mod" => {
                 let mut name = words[*it + 1].to_owned();
                 let ty = words[*it];
 
                 if *line + 1 < b_content.len() && b_content[*line].ends_with("::{") {
-                    move_to(&words, it, "\n", line, "");
-                    name.push_str(&b_content[*line + 1].trim());
+                    move_to(words, it, "\n", line, "");
+                    name.push_str(b_content[*line + 1].trim());
                 }
                 event_list.push(EventInfo::new(
                     *line,
                     EventType::Type(TypeStruct::new(Type::from(ty), &name)),
                 ));
             }
-            "struct" | "fn" | "enum" | "const" | "static" | "type" | "trait" | "macro_rules!"
+            "struct" | "enum" | "const" | "static" | "type" | "trait" | "macro_rules!"
             | "flags" => {
                 if *it + 1 >= words.len() {
                     break;
@@ -413,6 +436,32 @@ fn build_event_inner(
                 waiting_for_macro = words[*it] == "macro_rules!";
                 *it += 1;
             }
+            "fn" => {
+                if *it + 1 >= words.len() {
+                    break;
+                }
+                let name = get_before(words[*it + 1], STOP_CHARACTERS);
+                event_list.push(EventInfo::new(
+                    *line,
+                    EventType::Type(TypeStruct::new(Type::from(words[*it]), name)),
+                ));
+                *it += 1;
+                if !name.is_empty() {
+                    while let Some(&word) = words.get(*it) {
+                        if word.ends_with(';') {
+                            break;
+                        }
+                        if word.starts_with('{') {
+                            *it -= 1;
+                            break;
+                        }
+                        if word == "\n" {
+                            *line += 1;
+                        }
+                        *it += 1;
+                    }
+                }
+            }
             "!!" => {
                 event_list.push(EventInfo::new(
                     *line,
@@ -437,11 +486,20 @@ fn build_event_inner(
                     *line,
                     EventType::Type(TypeStruct::new(
                         Type::Impl,
-                        &join(&get_impl(&words, it, line), " "),
+                        &join(&get_impl(words, it, line), " "),
                     )),
                 ));
             }
-            "{" => {
+            c if c.starts_with("impl<") => {
+                event_list.push(EventInfo::new(
+                    *line,
+                    EventType::Type(TypeStruct::new(
+                        Type::Impl,
+                        &join(&get_impl(words, it, line), " "),
+                    )),
+                ));
+            }
+            x if x == "{" || x == "{\n" => {
                 if let Some(ref mut par_count) = par_count {
                     *par_count += 1;
                 }
@@ -450,16 +508,19 @@ fn build_event_inner(
                     build_event_inner(
                         it,
                         line,
-                        &words,
+                        words,
                         &mut vec![],
                         &mut vec![],
-                        &b_content,
+                        b_content,
                         Some(1),
                     );
                     waiting_for_macro = false;
                 }
+                if x == "}\n" {
+                    *line += 1;
+                }
             }
-            "}" => {
+            x if x == "}" || x == "}\n" => {
                 if let Some(ref mut par_count) = par_count {
                     *par_count -= 1;
                     if *par_count <= 0 {
@@ -467,6 +528,9 @@ fn build_event_inner(
                     }
                 }
                 event_list.push(EventInfo::new(*line, EventType::OutScope));
+                if x == "}\n" {
+                    *line += 1;
+                }
             }
             "\n" => {
                 *line += 1;
@@ -479,12 +543,17 @@ fn build_event_inner(
                     }
                     *it += 1;
                 }
+                *line += s.chars().filter(|c| *c == '\n').count();
             }
-            _ => {
+            x => {
                 event_list.push(EventInfo::new(
                     *line,
-                    EventType::Type(TypeStruct::new(Type::Unknown, words[*it])),
+                    EventType::Type(TypeStruct::new(
+                        Type::Unknown,
+                        &remove_stop_chars(words[*it]),
+                    )),
                 ));
+                *line += x.chars().filter(|c| *c == '\n').count();
             }
         }
         *it += 1;
@@ -512,8 +581,9 @@ pub fn build_event_list(path: &Path) -> io::Result<ParseResult> {
         &b_content,
         None,
     );
+    let clear = clear_events(event_list);
     Ok(ParseResult {
-        event_list: clear_events(event_list),
+        event_list: clear,
         comment_lines,
         original_content: b_content,
     })
@@ -585,13 +655,19 @@ pub fn strip_comments<F: Write>(
                             "{}\n",
                             &write_file_comment(&unformat_comment(c), &current, ignore_macros)
                         );
-                        while match parse_result.event_list[it].event {
-                            EventType::FileComment(ref c) => {
-                                comments.push_str(&format!("{}\n", unformat_comment(c)));
-                                true
-                            }
-                            _ => false,
-                        } {
+                        while parse_result
+                            .event_list
+                            .get(it)
+                            .map(|x| match x.event {
+                                EventType::FileComment(ref c) => {
+                                    use std::fmt::Write;
+                                    writeln!(comments, "{}", unformat_comment(c)).unwrap();
+                                    true
+                                }
+                                _ => false,
+                            })
+                            .unwrap_or(false)
+                        {
                             it += 1;
                         }
                         write!(out_file, "{}", comments).unwrap();
@@ -604,67 +680,71 @@ pub fn strip_comments<F: Write>(
                         while it < parse_result.event_list.len()
                             && match parse_result.event_list[it].event {
                                 EventType::Comment(ref c) => {
-                                    comments.push_str(&format!("{}\n", c));
+                                    use std::fmt::Write;
+                                    writeln!(comments, "{}", c).unwrap();
                                     true
                                 }
                                 EventType::Type(_) => false,
-                                _ => panic!("Doc comments cannot be written everywhere"),
+                                _ => panic!("[{}:{}]: Doc comments cannot be written everywhere:\n---> {:#?}", full_path.display(), parse_result.event_list[it].line, parse_result.event_list),
                             }
                         {
                             it += 1;
                         }
-                        if it >= parse_result.event_list.len() {
-                            continue;
-                        }
-                        while match parse_result.event_list[it].event {
-                            EventType::Type(ref t) => match t.ty {
-                                Type::Unknown => match current {
-                                    Some(ref cur) => {
-                                        if cur.ty == Type::Enum
-                                            || cur.ty == Type::Struct
-                                            || cur.ty == Type::Use
-                                        {
-                                            if t.name == "pub" {
-                                                true
-                                            } else {
-                                                let mut copy = t.clone();
-                                                copy.ty = Type::Variant;
-                                                let tmp = add_to_type_scope(&current, &Some(copy));
-                                                write!(
-                                                    out_file,
-                                                    "{}",
-                                                    write_comment(
-                                                        &tmp.unwrap(),
-                                                        &unformat_comment(&comments),
-                                                        ignore_macros
+                        while parse_result
+                            .event_list
+                            .get(it)
+                            .map(|x| match x.event {
+                                EventType::Type(ref t) => match t.ty {
+                                    Type::Unknown => match current {
+                                        Some(ref cur) => {
+                                            if cur.ty == Type::Enum
+                                                || cur.ty == Type::Struct
+                                                || cur.ty == Type::Use
+                                            {
+                                                if t.name == "pub" {
+                                                    true
+                                                } else {
+                                                    let mut copy = t.clone();
+                                                    copy.ty = Type::Variant;
+                                                    let tmp =
+                                                        add_to_type_scope(&current, &Some(copy));
+                                                    write!(
+                                                        out_file,
+                                                        "{}",
+                                                        write_comment(
+                                                            &tmp.unwrap(),
+                                                            &unformat_comment(&comments),
+                                                            ignore_macros
+                                                        )
                                                     )
-                                                )
-                                                .unwrap();
-                                                false
+                                                    .unwrap();
+                                                    false
+                                                }
+                                            } else {
+                                                t.name == "pub"
                                             }
-                                        } else {
-                                            t.name == "pub"
                                         }
-                                    }
-                                    None => t.name == "pub",
-                                },
-                                _ => {
-                                    let tmp = add_to_type_scope(&current, &Some(t.clone()));
-                                    write!(
-                                        out_file,
-                                        "{}",
-                                        write_comment(
-                                            &tmp.unwrap(),
-                                            &unformat_comment(&comments),
-                                            ignore_macros
+                                        None => t.name == "pub",
+                                    },
+                                    _ => {
+                                        let tmp = add_to_type_scope(&current, &Some(t.clone()));
+                                        write!(
+                                            out_file,
+                                            "{}",
+                                            write_comment(
+                                                &tmp.unwrap(),
+                                                &unformat_comment(&comments),
+                                                ignore_macros
+                                            )
                                         )
-                                    )
-                                    .unwrap();
-                                    false
-                                }
-                            },
-                            _ => panic!("An item was expected for this comment: {}", comments),
-                        } {
+                                        .unwrap();
+                                        false
+                                    }
+                                },
+                                _ => panic!("An item was expected for this comment: {}", comments),
+                            })
+                            .unwrap_or(false)
+                        {
                             it += 1;
                         }
                         continue;
@@ -686,10 +766,22 @@ pub fn strip_comments<F: Write>(
 }
 
 fn remove_comments(path: &Path, to_remove: &[usize], mut o_content: Vec<String>) {
+    let mut decal = 0;
     match File::create(path) {
         Ok(mut f) => {
-            for (decal, line) in to_remove.iter().enumerate() {
+            for line in to_remove.iter() {
+                if line - decal > 0
+                    && line - decal + 1 < o_content.len()
+                    && o_content[line - decal - 1].trim() == IGNORE_NEXT_COMMENT_STOP
+                {
+                    let l = o_content[line - decal + 1].trim();
+                    if DOC_COMMENT_ID.iter().any(|d| l.starts_with(d)) {
+                        o_content.remove(line - decal - 1);
+                        decal += 1;
+                    }
+                }
                 o_content.remove(line - decal);
+                decal += 1;
             }
             write!(f, "{}", o_content.join("\n")).unwrap();
         }
