@@ -145,7 +145,7 @@ fn test_regeneration() {
     stripper_lib::regenerate_doc_comments(
         temp_dir.path().to_str().unwrap(),
         false,
-        &temp_dir.path().join(comment_file).to_str().unwrap(),
+        temp_dir.path().join(comment_file).to_str().unwrap(),
         false,
         false,
     );
@@ -376,7 +376,7 @@ fn test2_regeneration() {
     stripper_lib::regenerate_doc_comments(
         temp_dir.path().to_str().unwrap(),
         false,
-        &temp_dir.path().join(comment_file).to_str().unwrap(),
+        temp_dir.path().join(comment_file).to_str().unwrap(),
         true,
         false,
     );
@@ -433,7 +433,7 @@ fn test3_regeneration() {
     stripper_lib::regenerate_doc_comments(
         temp_dir.path().to_str().unwrap(),
         false,
-        &temp_dir.path().join(comment_file).to_str().unwrap(),
+        temp_dir.path().join(comment_file).to_str().unwrap(),
         false,
         false,
     );
@@ -510,7 +510,7 @@ fn test4_regeneration() {
     stripper_lib::regenerate_doc_comments(
         temp_dir.path().to_str().unwrap(),
         false,
-        &temp_dir.path().join(comment_file).to_str().unwrap(),
+        temp_dir.path().join(comment_file).to_str().unwrap(),
         false,
         false,
     );
@@ -575,7 +575,7 @@ fn test5_regeneration() {
     stripper_lib::regenerate_doc_comments(
         temp_dir.path().to_str().unwrap(),
         false,
-        &temp_dir.path().join(comment_file).to_str().unwrap(),
+        temp_dir.path().join(comment_file).to_str().unwrap(),
         false,
         false,
     );
@@ -631,7 +631,7 @@ fn test6_regeneration() {
     stripper_lib::regenerate_doc_comments(
         temp_dir.path().to_str().unwrap(),
         false,
-        &temp_dir.path().join(comment_file).to_str().unwrap(),
+        temp_dir.path().join(comment_file).to_str().unwrap(),
         false,
         true,
     );
@@ -666,7 +666,7 @@ fn test7_regeneration() {
     stripper_lib::regenerate_doc_comments(
         temp_dir.path().to_str().unwrap(),
         false,
-        &temp_dir.path().join(comment_file).to_str().unwrap(),
+        temp_dir.path().join(comment_file).to_str().unwrap(),
         false,
         true,
     );
@@ -738,7 +738,7 @@ fn test8_regeneration() {
     stripper_lib::regenerate_doc_comments(
         temp_dir.path().to_str().unwrap(),
         false,
-        &temp_dir.path().join(comment_file).to_str().unwrap(),
+        temp_dir.path().join(comment_file).to_str().unwrap(),
         false,
         true,
     );
@@ -797,7 +797,7 @@ fn test9_regeneration() {
     stripper_lib::regenerate_doc_comments(
         temp_dir.path().to_str().unwrap(),
         false,
-        &temp_dir.path().join(comment_file).to_str().unwrap(),
+        temp_dir.path().join(comment_file).to_str().unwrap(),
         false,
         true,
     );
@@ -987,7 +987,7 @@ fn test10_regeneration() {
     stripper_lib::regenerate_doc_comments(
         temp_dir.path().to_str().unwrap(),
         false,
-        &temp_dir.path().join(comment_file).to_str().unwrap(),
+        temp_dir.path().join(comment_file).to_str().unwrap(),
         true,
         false,
     );
@@ -1005,7 +1005,7 @@ fn test10_regeneration2() {
     stripper_lib::regenerate_doc_comments(
         temp_dir.path().to_str().unwrap(),
         false,
-        &temp_dir.path().join(comment_file).to_str().unwrap(),
+        temp_dir.path().join(comment_file).to_str().unwrap(),
         false,
         true,
     );
@@ -1023,7 +1023,7 @@ fn test10_regeneration3() {
     stripper_lib::regenerate_doc_comments(
         temp_dir.path().to_str().unwrap(),
         false,
-        &temp_dir.path().join(comment_file).to_str().unwrap(),
+        temp_dir.path().join(comment_file).to_str().unwrap(),
         false,
         false,
     );
@@ -1041,7 +1041,7 @@ fn test10_regeneration4() {
     stripper_lib::regenerate_doc_comments(
         temp_dir.path().to_str().unwrap(),
         false,
-        &temp_dir.path().join(comment_file).to_str().unwrap(),
+        temp_dir.path().join(comment_file).to_str().unwrap(),
         true,
         true,
     );
@@ -1209,7 +1209,7 @@ fn test11_regeneration() {
     stripper_lib::regenerate_doc_comments(
         temp_dir.path().to_str().unwrap(),
         false,
-        &temp_dir.path().join(comment_file).to_str().unwrap(),
+        temp_dir.path().join(comment_file).to_str().unwrap(),
         true,
         false,
     );
@@ -1248,7 +1248,7 @@ mod foobar {
 }
 "#;
 
-fn get_basic12_md(file: &str) -> String {
+fn get_basic12_md(_file: &str) -> String {
     String::new()
 }
 
@@ -1349,9 +1349,559 @@ fn test13_regeneration() {
     stripper_lib::regenerate_doc_comments(
         temp_dir.path().to_str().unwrap(),
         false,
-        &temp_dir.path().join(comment_file).to_str().unwrap(),
+        temp_dir.path().join(comment_file).to_str().unwrap(),
         false,
         false,
     );
     compare_files(BASIC13_WEIRD, &temp_dir.path().join(test_file));
 }
+
+const BASIC14: &str = r#"
+/// Foo
+enum Foo {
+    /// Bar
+    Bar,
+    Blabla,
+    /// Toto
+    Toto,
+}
+"#;
+
+const BASIC14_STRIPPED: &str = r#"
+enum Foo {
+    Bar,
+    Blabla,
+    Toto,
+}
+"#;
+
+fn get_basic14_md(file: &str) -> String {
+    format!(
+        r#"<!-- file {} -->
+<!-- enum Foo -->
+Foo
+<!-- enum Foo::variant Bar -->
+Bar
+<!-- enum Foo::variant Toto -->
+Toto
+"#,
+        file
+    )
+}
+
+const BASIC14_MD: &str = r#"<!-- file * -->
+<!-- enum Foo -->
+Foo
+<!-- enum Foo::variant Bar -->
+Bar
+<!-- enum Foo::variant Toto -->
+Toto
+"#;
+
+#[allow(unused_must_use)]
+#[test]
+fn test14_strip_enum() {
+    let test_file = "basic14.rs";
+    let comment_file = "basic14.md";
+    let temp_dir = tempdir().unwrap();
+    gen_file(&temp_dir, test_file, BASIC14);
+    {
+        let mut f = gen_file(&temp_dir, comment_file, "");
+        stripper_lib::strip_comments(temp_dir.path(), test_file, &mut f, true);
+    }
+    compare_files(
+        &get_basic14_md(test_file),
+        &temp_dir.path().join(comment_file),
+    );
+    compare_files(BASIC14_STRIPPED, &temp_dir.path().join(test_file));
+}
+
+#[allow(unused_must_use)]
+#[test]
+fn test14_regeneration_enum() {
+    let test_file = "basic14.rs";
+    let comment_file = "basic14.md";
+    let temp_dir = tempdir().unwrap();
+    gen_file(&temp_dir, test_file, BASIC14_STRIPPED);
+    gen_file(&temp_dir, comment_file, BASIC14_MD);
+    stripper_lib::regenerate_doc_comments(
+        temp_dir.path().to_str().unwrap(),
+        false,
+        temp_dir.path().join(comment_file).to_str().unwrap(),
+        true,
+        false,
+    );
+    compare_files(BASIC14, &temp_dir.path().join(test_file));
+}
+
+const BASIC15_STRIPPED: &str = r#"
+// rustdoc-stripper-ignore-next
+/// Calls `gtk_widget_destroy()` on this widget.
+///
+/// # Safety
+///
+/// This will not necessarily entirely remove the widget from existence but
+/// you must *NOT* query the widget's state subsequently.  Do not call this
+/// yourself unless you really mean to.
+pub fn foo() {}
+
+// rustdoc-stripper-ignore-next
+/// lol
+pub fn bar() {}
+"#;
+
+const BASIC15: &str = r#"
+// rustdoc-stripper-ignore-next
+/// Calls `gtk_widget_destroy()` on this widget.
+///
+/// # Safety
+///
+/// This will not necessarily entirely remove the widget from existence but
+/// you must *NOT* query the widget's state subsequently.  Do not call this
+/// yourself unless you really mean to.
+// rustdoc-stripper-ignore-next-stop
+/// Utility function; intended to be connected to the `Widget::delete-event`
+/// signal on a `Window`. The function calls `WidgetExt::hide` on its
+/// argument, then returns `true`. If connected to ::delete-event, the
+/// result is that clicking the close button for a window (on the
+/// window frame, top right corner usually) will hide but not destroy
+/// the window. By default, GTK+ destroys windows when ::delete-event
+/// is received.
+///
+/// # Returns
+///
+/// `true`
+pub fn foo() {}
+
+// rustdoc-stripper-ignore-next
+/// lol
+pub fn bar() {}
+"#;
+
+const BASIC15_MD: &str = r#"<!-- fn foo -->
+Utility function; intended to be connected to the `Widget::delete-event`
+signal on a `Window`. The function calls `WidgetExt::hide` on its
+argument, then returns `true`. If connected to ::delete-event, the
+result is that clicking the close button for a window (on the
+window frame, top right corner usually) will hide but not destroy
+the window. By default, GTK+ destroys windows when ::delete-event
+is received.
+
+# Returns
+
+`true`
+"#;
+
+fn get_basic15_md(file: &str) -> String {
+    format!(
+        r#"<!-- file {} -->
+{}"#,
+        file, BASIC15_MD,
+    )
+}
+
+#[allow(unused_must_use)]
+#[test]
+fn test15_regeneration_ignore() {
+    let test_file = "basic15.rs";
+    let comment_file = "basic15.md";
+    let temp_dir = tempdir().unwrap();
+    gen_file(&temp_dir, test_file, BASIC15_STRIPPED);
+    gen_file(&temp_dir, comment_file, &get_basic15_md(test_file));
+    stripper_lib::regenerate_doc_comments(
+        temp_dir.path().to_str().unwrap(),
+        false,
+        temp_dir.path().join(comment_file).to_str().unwrap(),
+        true,
+        false,
+    );
+    compare_files(BASIC15, &temp_dir.path().join(test_file));
+}
+
+#[allow(unused_must_use)]
+#[test]
+fn test15_strip_ignore() {
+    let test_file = "basic15-strip.rs";
+    let comment_file = "basic15-strip.md";
+    let temp_dir = tempdir().unwrap();
+    gen_file(&temp_dir, test_file, BASIC15);
+    {
+        let mut f = gen_file(&temp_dir, comment_file, "");
+        stripper_lib::strip_comments(temp_dir.path(), test_file, &mut f, true);
+    }
+    compare_files(
+        &get_basic15_md(test_file),
+        &temp_dir.path().join(comment_file),
+    );
+    compare_files(BASIC15_STRIPPED, &temp_dir.path().join(test_file));
+}
+
+const BASIC16: &str = r#"impl Pixbuf {
+    pub fn from_mut_slice<T: AsMut<[u8]>>() -> Pixbuf {
+        let last_row_len = width * ((n_channels * bits_per_sample + 7) / 8);
+    }
+
+    // rustdoc-stripper-ignore-next
+    /// Creates a `Pixbuf` from a type implementing `Read` (like `File`).
+    ///
+    /// ```no_run
+    /// use std::fs::File;
+    /// use gdk_pixbuf::Pixbuf;
+    ///
+    /// let f = File::open("some_file.png").expect("failed to open image");
+    /// let pixbuf = Pixbuf::from_read(f).expect("failed to load image");
+    /// ```
+    pub fn from_read<R: Read + Send + 'static>(r: R) -> Result<Pixbuf, Error> {
+        Pixbuf::from_stream(&gio::ReadInputStream::new(r), None::<&gio::Cancellable>)
+    }
+}"#;
+
+#[allow(unused_must_use)]
+#[test]
+fn test16_strip_ignore() {
+    let test_file = "basic16-strip.rs";
+    let comment_file = "basic16-strip.md";
+    let temp_dir = tempdir().unwrap();
+    gen_file(&temp_dir, test_file, BASIC16);
+    {
+        let mut f = gen_file(&temp_dir, comment_file, "");
+        stripper_lib::strip_comments(temp_dir.path(), test_file, &mut f, true);
+    }
+    compare_files(BASIC16, &temp_dir.path().join(test_file));
+}
+
+// This test ensure we don't have an infinite loop in "strip::find_one_of".
+const BASIC17: &str = r#"
+pub const MIME_TYPE_JPEG: &str = "image/jpeg";
+pub const MIME_TYPE_PNG: &str = "image/png";
+pub const MIME_TYPE_JP2: &str = "image/jp2";
+pub const MIME_TYPE_URI: &str = "text/x-uri";"#;
+
+#[allow(unused_must_use)]
+#[test]
+fn test17_strip_ignore() {
+    let test_file = "basic17-strip.rs";
+    let comment_file = "basic17-strip.md";
+    let temp_dir = tempdir().unwrap();
+    gen_file(&temp_dir, test_file, BASIC17);
+    {
+        let mut f = gen_file(&temp_dir, comment_file, "");
+        stripper_lib::strip_comments(temp_dir.path(), test_file, &mut f, true);
+    }
+}
+
+const BASIC18: &str = r#"
+// Take a look at the license at the top of the repository in the LICENSE file.
+
+use crate::{resources_register, Resource};
+use glib::translate::*;
+use std::env;
+use std::mem;
+use std::path::Path;
+use std::process::Command;
+use std::ptr;
+
+impl Resource {
+    #[doc(alias = "g_resource_new_from_data")]
+    pub fn from_data(data: &glib::Bytes) -> Result<Resource, glib::Error> {
+        unsafe {
+            let mut error = ptr::null_mut();
+
+            // Create a copy of data if it is not pointer-aligned
+            // https://bugzilla.gnome.org/show_bug.cgi?id=790030
+            let mut data = data.clone();
+            let data_ptr = glib::ffi::g_bytes_get_data(data.to_glib_none().0, ptr::null_mut());
+            if data_ptr as usize % mem::align_of::<*const u8>() != 0 {
+                data = glib::Bytes::from(&*data);
+            }
+
+            let ret = ffi::g_resource_new_from_data(data.to_glib_none().0, &mut error);
+            if error.is_null() {
+                Ok(from_glib_full(ret))
+            } else {
+                Err(from_glib_full(error))
+            }
+        }
+    }
+}
+
+/// Call from build script to run `glib-compile-resources` to generate compiled gresources to embed
+/// in binary with [resources_register_include]. `target` is relative to `OUT_DIR`.
+///
+/// ```no_run
+/// gio::compile_resources(
+///     "resources",
+///     "resources/resources.gresource.xml",
+///     "compiled.gresource",
+/// );
+/// ```
+pub fn compile_resources<P: AsRef<Path>>(source_dir: P, gresource: &str, target: &str) {
+    let out_dir = env::var("OUT_DIR").unwrap();
+
+    let status = Command::new("glib-compile-resources")
+        .arg("--sourcedir")
+        .arg(source_dir.as_ref())
+        .arg("--target")
+        .arg(&format!("{}/{}", out_dir, target))
+        .arg(gresource)
+        .status()
+        .unwrap();
+
+    if !status.success() {
+        panic!("glib-compile-resources failed with exit status {}", status);
+    }
+
+    println!("cargo:rerun-if-changed={}", gresource);
+    let output = Command::new("glib-compile-resources")
+        .arg("--sourcedir")
+        .arg(source_dir.as_ref())
+        .arg("--generate-dependencies")
+        .arg(gresource)
+        .output()
+        .unwrap()
+        .stdout;
+    let output = String::from_utf8(output).unwrap();
+    for dep in output.split_whitespace() {
+        println!("cargo:rerun-if-changed={}", dep);
+    }
+}
+
+#[doc(hidden)]
+pub fn resources_register_include_impl(bytes: &'static [u8]) -> Result<(), glib::Error> {
+    let bytes = glib::Bytes::from_static(bytes);
+    let resource = Resource::from_data(&bytes)?;
+    resources_register(&resource);
+    Ok(())
+}
+
+/// Include gresources generated with [compile_resources] and register with glib. `path` is
+/// relative to `OUTDIR`.
+///
+/// ```ignore
+/// gio::resources_register_include!("compiled.gresource").unwrap();
+/// ```
+#[macro_export]
+macro_rules! resources_register_include {
+    ($path:expr) => {
+        $crate::resources_register_include_impl(include_bytes!(concat!(
+            env!("OUT_DIR"),
+            "/",
+            $path
+        )))
+    };
+}
+"#;
+
+const BASIC18_STRIPPED: &str = r#"
+// Take a look at the license at the top of the repository in the LICENSE file.
+
+use crate::{resources_register, Resource};
+use glib::translate::*;
+use std::env;
+use std::mem;
+use std::path::Path;
+use std::process::Command;
+use std::ptr;
+
+impl Resource {
+    #[doc(alias = "g_resource_new_from_data")]
+    pub fn from_data(data: &glib::Bytes) -> Result<Resource, glib::Error> {
+        unsafe {
+            let mut error = ptr::null_mut();
+
+            // Create a copy of data if it is not pointer-aligned
+            // https://bugzilla.gnome.org/show_bug.cgi?id=790030
+            let mut data = data.clone();
+            let data_ptr = glib::ffi::g_bytes_get_data(data.to_glib_none().0, ptr::null_mut());
+            if data_ptr as usize % mem::align_of::<*const u8>() != 0 {
+                data = glib::Bytes::from(&*data);
+            }
+
+            let ret = ffi::g_resource_new_from_data(data.to_glib_none().0, &mut error);
+            if error.is_null() {
+                Ok(from_glib_full(ret))
+            } else {
+                Err(from_glib_full(error))
+            }
+        }
+    }
+}
+
+pub fn compile_resources<P: AsRef<Path>>(source_dir: P, gresource: &str, target: &str) {
+    let out_dir = env::var("OUT_DIR").unwrap();
+
+    let status = Command::new("glib-compile-resources")
+        .arg("--sourcedir")
+        .arg(source_dir.as_ref())
+        .arg("--target")
+        .arg(&format!("{}/{}", out_dir, target))
+        .arg(gresource)
+        .status()
+        .unwrap();
+
+    if !status.success() {
+        panic!("glib-compile-resources failed with exit status {}", status);
+    }
+
+    println!("cargo:rerun-if-changed={}", gresource);
+    let output = Command::new("glib-compile-resources")
+        .arg("--sourcedir")
+        .arg(source_dir.as_ref())
+        .arg("--generate-dependencies")
+        .arg(gresource)
+        .output()
+        .unwrap()
+        .stdout;
+    let output = String::from_utf8(output).unwrap();
+    for dep in output.split_whitespace() {
+        println!("cargo:rerun-if-changed={}", dep);
+    }
+}
+
+#[doc(hidden)]
+pub fn resources_register_include_impl(bytes: &'static [u8]) -> Result<(), glib::Error> {
+    let bytes = glib::Bytes::from_static(bytes);
+    let resource = Resource::from_data(&bytes)?;
+    resources_register(&resource);
+    Ok(())
+}
+
+#[macro_export]
+macro_rules! resources_register_include {
+    ($path:expr) => {
+        $crate::resources_register_include_impl(include_bytes!(concat!(
+            env!("OUT_DIR"),
+            "/",
+            $path
+        )))
+    };
+}
+"#;
+
+const BASIC18_MD: &str = r#"<!-- fn compile_resources -->
+Call from build script to run `glib-compile-resources` to generate compiled gresources to embed
+in binary with [resources_register_include]. `target` is relative to `OUT_DIR`.
+
+```no_run
+gio::compile_resources(
+    "resources",
+    "resources/resources.gresource.xml",
+    "compiled.gresource",
+);
+```
+<!-- macro resources_register_include -->
+Include gresources generated with [compile_resources] and register with glib. `path` is
+relative to `OUTDIR`.
+
+```ignore
+gio::resources_register_include!("compiled.gresource").unwrap();
+```
+"#;
+
+fn get_basic18_md(file: &str) -> String {
+    format!(
+        r#"<!-- file {} -->
+{}"#,
+        file, BASIC18_MD,
+    )
+}
+
+#[allow(unused_must_use)]
+#[test]
+fn test18_strip_failure() {
+    let test_file = "basic18-strip.rs";
+    let comment_file = "basic18-strip.md";
+    let temp_dir = tempdir().unwrap();
+    gen_file(&temp_dir, test_file, BASIC18);
+    {
+        let mut f = gen_file(&temp_dir, comment_file, "");
+        stripper_lib::strip_comments(temp_dir.path(), test_file, &mut f, true);
+    }
+    compare_files(
+        &get_basic18_md(test_file),
+        &temp_dir.path().join(comment_file),
+    );
+    compare_files(BASIC18_STRIPPED, &temp_dir.path().join(test_file));
+}
+
+#[test]
+fn item_inside_macro() {
+    static SRC_RS: &str = r###"
+        glib::wrapper! {
+            pub struct BaseInfo: u32 {
+                const READABLE = 5
+            };
+        }
+    "###;
+    static DOCS_MD: &str = r###"<!-- file * -->
+<!-- struct BaseInfo -->
+GIBaseInfo is the common base struct of all other Info structs
+accessible through the [`Repository`][crate::Repository] API.
+<!-- struct BaseInfo::const READABLE -->
+This thing can be read.
+"###;
+    static TARGET_RS: &str = r###"
+        glib::wrapper! {
+            /// GIBaseInfo is the common base struct of all other Info structs
+            /// accessible through the [`Repository`][crate::Repository] API.
+            pub struct BaseInfo: u32 {
+                /// This thing can be read.
+                const READABLE = 5
+            };
+        }
+    "###;
+
+    let src_path = "iim.rs";
+    let docs_path = "iim-docs.md";
+    let temp_dir = tempdir().unwrap();
+    gen_file(&temp_dir, src_path, SRC_RS);
+    gen_file(&temp_dir, docs_path, DOCS_MD);
+
+    stripper_lib::regenerate_doc_comments(
+        temp_dir.path().to_str().unwrap(),
+        false,
+        temp_dir.path().join(docs_path).to_str().unwrap(),
+        false,
+        false,
+    );
+    compare_files(TARGET_RS, &temp_dir.path().join(src_path));
+}
+
+#[test]
+fn fn_with_impl_arg_and_ret() {
+    static SRC_RS: &str = r###"
+        trait ObjectExt {
+            fn impl_arg(&self, t: impl Trait) -> i32;
+            fn impl_no_arg(&self) -> i32;
+        }
+    "###;
+    static DOCS_MD: &str = r###"<!-- file * -->
+<!-- trait ObjectExt::fn impl_arg  -->
+A function that takes an impl trait arg.
+<!-- trait ObjectExt::fn impl_no_arg  -->
+A function that takes no arguments.
+"###;
+    static TARGET_RS: &str = r###"
+        trait ObjectExt {
+            /// A function that takes an impl trait arg.
+            fn impl_arg(&self, t: impl Trait) -> i32;
+            /// A function that takes no arguments.
+            fn impl_no_arg(&self) -> i32;
+        }
+    "###;
+
+    let src_path = "fnimpl.rs";
+    let docs_path = "fnimpl-docs.md";
+    let temp_dir = tempdir().unwrap();
+    gen_file(&temp_dir, src_path, SRC_RS);
+    gen_file(&temp_dir, docs_path, DOCS_MD);
+
+    stripper_lib::regenerate_doc_comments(
+        temp_dir.path().to_str().unwrap(),
+        false,
+        temp_dir.path().join(docs_path).to_str().unwrap(),
+        false,
+        false,
+    );
+    compare_files(TARGET_RS, &temp_dir.path().join(src_path));
+}
